/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/*
**  swing_27 - remote export import
*/
#include <stdio.h>
#include <ssdef.h>
#include <smgdef.h>
#include <smgmsg.h>
#include <syidef.h>
#include <lnmdef.h>
#include <psldef.h>
#include <trmdef.h>
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

#define DISPLAY_REMOTE_ROWS	8
#define DISPLAY_REMOTE_COLUMNS	76
#define DISPLAY_REMOTE_X	2
#define DISPLAY_REMOTE_Y	14

#define NODE_X		1
#define NODE_Y		1
#define DEVICE_X	1
#define DEVICE_Y	2
#define DIRECTORY_X	1
#define DIRECTORY_Y	3
#define FILENAME_X	1
#define FILENAME_Y	4
#define RSTATUS_X	1
#define RSTATUS_Y	5
#define ACTION_X	1
#define ACTION_Y	7

#define DATA_OFFSET	14

static int remote_initialized = 0;
static int display_remote;
static char this_node[SPEC_SIZE+1];
globalref int captive;
/*
**  get the node name of this node (if there is one)
*/
get_node_name(output)
char *output;
{
    int i, return_length, acmode;
    char node_name[21];

    struct {
    short buffer_length;
    short item_code;
    int buffer_address;
    int return_length_address;
    int terminator;
    } syitmlst[1] = { 20,	SYI$_NODENAME,	node_name,
                      &return_length,	0};

    struct {
    short buffer_length;
    short item_code; 
    int buffer_address;
    int return_length_address;
    int terminator;
    } lnmitmlst[1] = { 15,	LNM$_STRING,	node_name,	
                       &return_length,	0};

    node_name[0] = 0;
    i = sys$getsyiw(0,0,0,&syitmlst[0].buffer_length,0,0,0);
    if (i != SS$_NORMAL)
    {
        panic(i,"Getting this_node via sys$getsyiw","remote_init");
    }
    
    if (node_name[0] == 0)
    {
        acmode  = PSL$C_EXEC;
        sys$trnlnm(&LNM$M_CASE_BLIND,
    	           DSC1("LNM$SYSTEM_TABLE"),
    	           DSC2("SYS$NODE"),
    	           &acmode,
                   &lnmitmlst[0].buffer_length);
    }

    i = strlen(node_name);
    if (i)
    {
        /*
        **  make sure we're terminated by two colons as per all good node 
	**  names
        */
        if (node_name[i-2] != ':')
            strcat(node_name,":");
        if (node_name[i-1] != ':')
            strcat(node_name,":");
    }            

    strcpy(output,node_name);
}

/*
**  initialisation function
*/
remote_init(dir)
char *dir;
{
    int i, rows, columns;
    
    /*
    **  the display
    */
    rows    = DISPLAY_REMOTE_ROWS;
    columns = DISPLAY_REMOTE_COLUMNS;
    i = smg$create_virtual_display(&rows,
                                   &columns,
                                   &display_remote,
                                   &SMG$M_BORDER);
    if (i != SS$_NORMAL)
        panic(i,"create virtual display display_remote","remote_init");

    /*
    **  find this node name
    */
    get_node_name(this_node);
    strcpy(remote_node,this_node);

    if (!remote_device[0])
        translate("SYS$DISK",remote_device);
    if (!remote_directory[0])
        strcpy(remote_directory,&dir[strcspn(dir,":")+1]);

    remote_initialized = 1;
}

/*
**  display a line on the display, line is title(tx,ty) + data(dx,dy)
*/
remote_display_line(title,tx,ty,data,dx,dy)
char *title, *data;
int tx,ty,dx,dy;
{
    smg$set_cursor_abs(&display_remote,&ty,&tx);
    smg$put_line(&display_remote,DSC1(title));

    smg$set_cursor_abs(&display_remote,&dy,&dx);
    if (strlen(data))
        smg$put_line(&display_remote,DSC1(data),&0,&SMG$M_UNDERLINE);
    else
        smg$put_line(&display_remote,DSC1(" "),&0,&0);
}

/*
**  entry point for remote file operations
*/
remote_files(dir,file_count,option)
char *dir;
int file_count;
int option;
{
    int i, screen;

    char destination[SPEC_SIZE+SPEC_SIZE+1];
    char new_filename[SPEC_SIZE+SPEC_SIZE+1];

    if (captive)
    {
	message(CAPTIVE_ERRMSG);
	return(FALSE);
    }
    /*
    **  if we're exporting and we haven't selected a file try again  
    */
    if (option == EXPORT && !file_count)
    {
        message("No files selected");
        return(FALSE);
    }

    /*
    **  save and clear current display
    */
    smg$save_physical_screen(&paste, &screen, &1, &23);

    /*
    **  call initialisation routine if not done so already
    */
    if (!remote_initialized)
        remote_init(dir);

    /*
    **  clear out this display
    */
    i = smg$erase_display(&display_remote);
    if (i != SS$_NORMAL)
        panic(i,"erase display","remote_files");

    /*
    **  decide on an advertising logo
    */
    if (option == EXPORT)
    {
        i = smg$label_border(&display_remote,DSC1(" FILE EXPORT "),
                             &SMG$K_TOP,0,&(SMG$M_REVERSE));
    }
    else
    {
        i = smg$label_border(&display_remote,DSC1(" FILE IMPORT "),
                             &SMG$K_TOP,0,&(SMG$M_REVERSE));
    }
    if (i != SS$_NORMAL)
        panic(i,"label border display_remote","remote_init");

    /*
    **  node
    */
    remote_display_line(" NODE       : ",
                        NODE_X,
                        NODE_Y,
                        remote_node,
                        NODE_X+DATA_OFFSET,
                        NODE_Y);

    /*
    **  device
    */
    remote_display_line(" DEVICE     : ",
                        DEVICE_X,
                        DEVICE_Y,
                        remote_device,
                        DEVICE_X+DATA_OFFSET,
                        DEVICE_Y);

    /*
    **  directory
    */
    remote_display_line(" DIRECTORY  : ",
                        DIRECTORY_X,
                        DIRECTORY_Y,
                        remote_directory,
                        DIRECTORY_X+DATA_OFFSET,
                        DIRECTORY_Y);

    /*
    **  filename
    */
    if (option == EXPORT)
    {
        remote_display_line(" FILENAME   : ",
                            FILENAME_X,
                            FILENAME_Y,
                            "* UNCHANGED *",
                            FILENAME_X+DATA_OFFSET,
                            FILENAME_Y);
    }
    else
    {
        remote_display_line(" FILENAME   : ",
                            FILENAME_X,
                            FILENAME_Y,
                            remote_filename,
                            FILENAME_X+DATA_OFFSET,
                            FILENAME_Y);
    }


    /*
    **  status
    */
    remote_display_line(" STATUS     : ",
                        RSTATUS_X,
                        RSTATUS_Y,
                        "",
                        RSTATUS_X+DATA_OFFSET,
                        RSTATUS_Y);

    /*
    **  let viewer see the program
    */
    i = smg$paste_virtual_display(&display_remote,
                                  &paste,
                                  &DISPLAY_REMOTE_Y,
                                  &DISPLAY_REMOTE_X);
    if (i != SS$_NORMAL)
        panic(i,"paste","remote_files");

    /*
    **  now find out what the user whishes to do
    */
    while(1)
    {    
        switch(remote_get_option(" What now - C(opy), M(odify), Q(uit) ?"))
        {
            /*
            **  copy files
            */
    case SMG$K_TRM_UPPERCASE_C:
    case SMG$K_TRM_LOWERCASE_C:
            /*
            **  build up a full file specification
            */
            strcpy(destination,remote_node);
            strcat(destination,remote_device);
            strcat(destination,remote_directory);
            if (option == EXPORT)
            {
                /*
                **  export the selected files
                */
                remote_copy(dir,destination);
            }
            else
            {
                /*
                **  import the one and only file - user must specify it
                */
                if (!remote_filename[0])
                {
                    remote_status_line("No filename specified");
                    break;
                }

                /*
                **  add filename into the specification
                */
                strcat(destination,remote_filename);

                /*
                **  establish where we'll put the file
                */
                sprintf(new_filename,"%s%s",dir,remote_filename);

                /*
                **  call the copy function
                */
                i = lib_copy_file(destination,
                                  new_filename,
                                  remote_filename,
                                  0,
                                  IMPORT,
                                  0);

                /*
                **  say if we had problems
                */
                if (i != SS$_NORMAL)
                {
                    sprintf(junk,"%s - failed to copy %s",
                    error_text(i),remote_filename);
                    remote_status_line(junk);
                    warning(full_error_text(i));
                }
                else
                    remote_status_line(full_error_text(i));

                /*
                **  close down the copy window
                */
                copy_window_off(TRUE);

            }
            break;

            /*
            **  quit
            */
    case SMG$K_TRM_UPPERCASE_Q:
    case SMG$K_TRM_LOWERCASE_Q:
            /*
            **  clear away the display
            */
            smg$unpaste_virtual_display(&display_remote,&paste);

            /*
            **  restore what was there before
            */
            smg$restore_physical_screen(&paste,&screen);
            return(TRUE);
            break;
    
            /*
            **  modify specifications
            */
    case SMG$K_TRM_UPPERCASE_M:
    case SMG$K_TRM_LOWERCASE_M:
            /*
            **  clear down the status line
            */
            remote_status_line("");

            /*
            **  now get a node, device, directory and possibly a filename
            */
            remote_get_string(remote_node,
                              NODE_Y,
                              NODE_X+DATA_OFFSET);
            remote_get_string(remote_device,
                              DEVICE_Y,
                              DEVICE_X+DATA_OFFSET);
            remote_get_string(remote_directory,
                              DIRECTORY_Y,
                              DIRECTORY_X+DATA_OFFSET);
            if (option == IMPORT)
                remote_get_string(remote_filename,
                                  FILENAME_Y,
                                  FILENAME_X+DATA_OFFSET);

            break;
    
            /*
            **  tyyping trouble
            */
    default:
            remote_status_line("Invalid input");
            break;
        }
    }
}

remote_status_line(text)
char *text;
{
    smg$set_cursor_abs(&display_remote,&RSTATUS_Y,&(RSTATUS_X+DATA_OFFSET));
    smg$put_line(&display_remote,DSC1(text),&1,0);
    smg$ring_bell(&displayb);
}

/*
**  get a string at y,x
*/
remote_get_string(text,y,x)
char *text;
int x,y;
{
    int i,modifiers,return_length=0;
    char tmp[SPEC_SIZE+1];

    /*
    **  force uppercase, no echo, no recall
    */
    modifiers = TRM$M_TM_CVTLOW + TRM$M_TM_TRMNOECHO +
                TRM$M_TM_NORECALL;

    smg$set_cursor_abs(&display_remote,&y,&x);
    smg$read_string(&keyboard,
                    DSCA1(tmp),
                    DSC1(""),
                    0,
                    &modifiers,
                    0,
                    0,
                    &return_length,
                    0,
                    &display_remote,
                    DSC2(text));

    tmp[return_length] = 0;
    /*
    **  strip off trailing spaces
    */
    for (i = (strlen(tmp)-1); i >= 0; i--)
       if (tmp[i] != ' ')
           break;
       else
           tmp[i] = 0;

    /*
    **  redisplay, this time underlined
    */
    smg$set_cursor_abs(&display_remote,&y,&x);
    smg$put_line(&display_remote,DSC1(tmp),&0,&SMG$M_UNDERLINE);

    /*
    ** heave-ho into the output
    */
    strcpy(text,tmp);
}

/*
**  export all the specified files
*/
remote_copy(from_dir,to)
char *from_dir, *to;
{
    int i, remaining;
    char from_spec[SPEC_SIZE+1];
    char to_spec[SPEC_SIZE+1];
    FILER *temp;

    /*
    **  count of uncopied files still to be copied - used in copy window
    */
    remaining = files_selected;

    /*
    **  walk down the linked list
    */
    temp = first_file_ptr;
    while(temp)
    {
        /*
        **  if the rendition is on we've chosen it
        */
        if (temp->rendition)
        {
            /*
            **  yup, this one's been chosen
            */
            sprintf(from_spec,"%s%s",from_dir,temp->filename);
            sprintf(to_spec,"%s%s",to,temp->filename);
            /*
            **  now strip off the version number of the destination,
            **  so that it is copied to be the newest version of this
            **  file in the destination dir
            */
            for (i = strlen(to_spec) - 1; i > 0; i--)
            {
                if (to_spec[i] == ';')
                {
                    to_spec[i] = 0;
                    break;
                }
            }

            /*
            **  call the copy function
            */ 
            i = lib_copy_file(from_spec,
                              to_spec,
                              temp->filename,
                              --remaining,
                              EXPORT,
                              0);

            if (i == SS$_NORMAL)
                remote_status_line(full_error_text(i));
            else
            {
                /*
                **  error
                */
                sprintf(junk,"%s - failed to copy %s",
                error_text(i),temp->filename);
                remote_status_line(junk);
                warning(full_error_text(i));

                /*
                **  see if user wishes to continue
                */
                while(1)
                {
                    switch(remote_get_option(" Continue - Y(es), N(o) ? "))
                    {
			/*
                        **  no more copying
                        */
                case SMG$K_TRM_LOWERCASE_N:
                case SMG$K_TRM_UPPERCASE_N:
                        /*
                        **  off with the copy window
                        */
                        copy_window_off(FALSE);
                        return(FALSE);

                case SMG$K_TRM_LOWERCASE_Y:
                case SMG$K_TRM_UPPERCASE_Y:
                        /*
                        **  let's have another go
                        */
                        goto out_of_loop;

                        /*
                        **  duff input
                        */
                default:
                        break;
                    }
                }
            out_of_loop:;
            }
        }
        /*
        **  point to next file
        */
        temp = temp->forw_ptr;    
    }

    /*
    **  off with the copy window
    */
    copy_window_off(TRUE);
}

/*
**  get input from the user, just one char will do
*/
remote_get_option(prompt)
char *prompt;
{
    int i;
    short key;

    while(1)
    {
        smg$set_cursor_abs(&display_remote,&ACTION_Y,&ACTION_X);
        i = smg$read_keystroke(&keyboard, 
                               &key, 
                               DSC2(prompt), 
                               0,
                               &display_remote);
        if (i != SS$_NORMAL)
            panic(i,"read keystroke","remote get option");

        /*
        **  refresh
        */
        if (key == SMG$K_TRM_CTRLW || key == SMG$K_TRM_CTRLR)
            smg$repaint_screen(&paste);
        else if (key == SMG$K_TRM_ASTERISK)
	    snapshot();
	else	
        {
            smg$erase_line(&display_remote,&ACTION_Y,&ACTION_X);
            return(key);
        }
    }
}
